@@ -19,6 +19,9 @@
  */
 
 #include "kxmleditorshell.h"
+#include "kxeshellmanager.h"
+#include "../part/kxmleditorpart.h"
+#include "../part/kxedocument.h"
 
 #include <klibloader.h>
 #include <kstdaction.h>
@@ -30,10 +33,17 @@
 #include <kedittoolbar.h>
 #include <kstatusbar.h>
 #include <kparts/browserextension.h>
-#include "../part/kxmleditorpart.h"
+#include <kparts/partmanager.h>
+#include <kmessagebox.h>
+
+#include <kpopupmenu.h>
+
+#include <qevent.h>
+
 #include "dcopiface_shell.h"
 
-#define FILE_DIALOG_FILTER "*.xml|XML files\n*.xsl|XSL files\n*.xslt|XSLT files\n*.xul|XUL files\n*.rc|Resource UI files\n*.ui|User Interface UI files\n*|All files"
+
+KXEShellManager *KXMLEditorShell::s_manager=0L;
 
 KXMLEditorShell::KXMLEditorShell( const char * name, WFlags f )
 	: KParts::MainWindow( name, f ),
@@ -43,56 +53,31 @@ KXMLEditorShell::KXMLEditorShell( const char * name, WFlags f )
 	//////////////////////////////
 	// CREATE ACTIONS
 	//////////////////////////////
+
 	setXMLFile("kxmleditorshell.rc");
 
 	KStdAction::openNew(this,SLOT(slotActFileNew()),actionCollection(),"file_new");
+	
 	KStdAction::open( this, SLOT(slotFileOpen()), actionCollection(),"file_open" );
+	
 	m_pActFileOpenRecent = KStdAction::openRecent( this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection(),"file_open_recent");
 	m_pActFileOpenRecent->loadEntries( instance()->config() );
-	m_pActFileSave = KStdAction::save( this, SLOT(slotFileSave()), actionCollection(),"file_save" );
-	KStdAction::saveAs( this, SLOT(slotFileSaveAs()), actionCollection() ,"file_save_as");
-	m_pActFileSave->setEnabled(false);
-
-	KToggleAction * pActToggleToolBar = KStdAction::showToolbar( this, SLOT(slotToggleToolBar()), actionCollection() );
+	
+	KToggleAction * pActToggleMainToolBar = KStdAction::showToolbar( this, SLOT(slotToggleMainToolBar()), actionCollection() );
 	KToggleAction * pActToggleStatusBar = KStdAction::showStatusbar( this, SLOT(slotToggleStatusBar()), actionCollection() );
 	KStdAction::keyBindings( this, SLOT(slotConfigureKeys()), actionCollection() );
 	KStdAction::configureToolbars( this, SLOT(slotConfigureToolbars()), actionCollection() );
 
-        KStdAction::mail(this,SLOT(mail()),actionCollection(),"file_mail");
+  KStdAction::mail(this,SLOT(mail()),actionCollection(),"mail");
+  KStdAction::close(this,SLOT(slotActClose()),actionCollection(),"close");
 
-	KStdAction::quit( this, SLOT(close()), actionCollection(),"file_quit");
+	KStdAction::quit( this, SLOT(close()), actionCollection(),"quit");
 
-	// try to find libkxmleditor
-	QString szLibName("libkxmleditorpart");
-	KLibFactory * pFactory = KLibLoader::self()->factory(szLibName);
-	if (pFactory)
-	{
-		m_pPart = static_cast <KParts::ReadWritePart*> ( pFactory->create( this, "KXMLEditor view", "KParts::ReadWritePart" ) );
-		setCentralWidget( m_pPart->widget() );
+	// L.V. moved to part pActPrint = KStdAction::print( 0, 0, actionCollection(), "print" );
 
-		// get the parts browser extension and initialize additional actions with it
-		KParts::BrowserExtension * pBrowserExt = 0;
-		if ( (pBrowserExt = KParts::BrowserExtension::childObject(m_pPart)) )  // if there is one
-		{
-			connect( pBrowserExt, SIGNAL(enableAction( const char *, bool )), this, SLOT(slotEnableBrowserExtActions( const char *, bool )) );
-
-			KAction * pActEditCut = KStdAction::cut( pBrowserExt, SLOT(cut()), actionCollection(), "cut" );
-			pActEditCut->setEnabled(pBrowserExt->isActionEnabled("cut"));
-
-			KAction * pActEditCopy = KStdAction::copy( pBrowserExt, SLOT(copy()), actionCollection(), "copy" );
-			pActEditCopy->setEnabled(pBrowserExt->isActionEnabled("copy"));
-
-			KAction * pActEditPaste = KStdAction::paste( pBrowserExt, SLOT(paste()), actionCollection(), "paste" );
-			pActEditPaste->setEnabled(pBrowserExt->isActionEnabled("paste"));
-
-			KAction * pActPrint = KStdAction::print( pBrowserExt, SLOT(print()), actionCollection(), "print" );
-			pActPrint->setEnabled(pBrowserExt->isActionEnabled("print"));
-		}
-
-		createGUI( m_pPart );
-	}
-	else
-		kdFatal() << "KXMLEditorShell::KXMLEditorShell no " << szLibName << " found" << endl;
+	pActCloseW = new KAction( i18n("&Close"), 0, 0, this,SLOT(slotActWindowClose()), actionCollection(), "window_close" );
+	pActCloseAllW = new KAction( i18n("Close &all"), 0, 0, this,SLOT(slotActWindowCloseAll()), actionCollection(), "window_close_all" );
+	pActCloseAllO = new KAction( i18n("Close all &others"), 0, 0, this,SLOT(slotActWindowCloseAllOthers()), actionCollection(), "window_close_all_others" );
 
 	//////////////////////////////
 	// CREATE STATUSBAR
@@ -102,19 +87,27 @@ KXMLEditorShell::KXMLEditorShell( const char * name, WFlags f )
 	//////////////////////////////
 	// MISC. INITIALIZATION
 	//////////////////////////////
+
+	manager()->addShell(this);
+
 	setAutoSaveSettings();
-	pActToggleToolBar->setChecked( ! toolBar()->isHidden() );
+	pActToggleMainToolBar->setChecked( ! toolBar()->isHidden() );
 	pActToggleStatusBar->setChecked( ! statusBar()->isHidden() );
 
 	m_pKXEShellIface = new KXMLEditorShellIface(*this);
 
 	statusBar()->setSizeGripEnabled(true);
+
+	setPart(0L);		// start empty
+
 }
 
 KXMLEditorShell::~KXMLEditorShell()
 {
 	if (m_pKXEShellIface)
 		delete m_pKXEShellIface;
+
+	manager()->removeShell(this);
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -129,7 +122,9 @@ void KXMLEditorShell::slotFileOpen()
 	                                    i18n(FILE_DIALOG_FILTER),
 	                                    this );
 	openURL(url);
-  m_pActFileSave->setEnabled(true);
+	
+	// L.V.: tried to meve save functionality to Part
+	//m_pActFileSave->setEnabled(true); // TODO: L.V. Why File Save ? Why not Save as... ?
 
 	statusBar()->message( i18n("Ready.") );
 }
@@ -143,57 +138,7 @@ void KXMLEditorShell::slotFileOpenRecent(const KURL & url)
 	statusBar()->message( i18n("Ready.") );
 }
 
-void KXMLEditorShell::slotFileSave()
-{
-	statusBar()->message( i18n("Saving file...") );
-
-	if( m_pPart->url().isEmpty() )
-		slotFileSaveAs();
-	else
-		m_pPart->save();
-
-	statusBar()->message( i18n("Ready.") );
-}
-
-void KXMLEditorShell::slotFileSaveAs()
-{
-	statusBar()->message( i18n("Saving file with a new filename...") );
-
-	KFileDialog dlg( QDir::currentDirPath(),   // start dir.
-	                 i18n(FILE_DIALOG_FILTER), // filter
-	                 this,                     // parent
-	                 "file dialog for saving", // name
-	                 true );                   // modal
-	dlg.setCaption( i18n("Save as...") );
-	dlg.setOperationMode( KFileDialog::Saving );
-	dlg.exec();
-
-	KURL url = dlg.selectedURL();
-
-	if(!url.isEmpty())
-	{
-    // append filetype if necessary
-    if(dlg.currentFilter() != "*.*")
-    { QString strExtension = dlg.currentFilter();
-      strExtension.remove('*');
-
-      if(strExtension != url.fileName().right(strExtension.length()))
-        url.setFileName(url.fileName() + strExtension );
-    }
-
-		if (m_pPart->saveAs(url))	// something may go wrong
-		{
-			m_pActFileOpenRecent->addURL(url);
-			m_pPart->setModified(false);
-
-			m_pActFileSave->setEnabled(false);
-		}
-	}
-
-	statusBar()->message( i18n("Ready.") );
-}
-
-void KXMLEditorShell::slotToggleToolBar()
+void KXMLEditorShell::slotToggleMainToolBar()
 {
 	statusBar()->message( i18n("Toggle the toolbar...") );
 
@@ -205,6 +150,7 @@ void KXMLEditorShell::slotToggleToolBar()
 	statusBar()->message( i18n("Ready.") );
 }
 
+
 void KXMLEditorShell::slotToggleStatusBar()
 {
 	statusBar()->message( i18n("Toggle the statusbar...") );
@@ -220,7 +166,7 @@ void KXMLEditorShell::slotToggleStatusBar()
 void KXMLEditorShell::slotConfigureKeys()
 {
 	KKeyDialog dlg;
-	dlg.insert(m_pPart->actionCollection());
+	dlg.insert(part()->actionCollection());
 	dlg.insert(actionCollection());
 	dlg.configure();
 }
@@ -229,14 +175,14 @@ void KXMLEditorShell::slotConfigureToolbars()
 {
 	KEditToolbar dlg( factory() );
 	if ( dlg.exec() )
-		createGUI( m_pPart );
+		createGUI( part() );
 }
 
 /////////////////////////////////////////////////////////////////////
 //                    ADDITIONAL FUNCTIONS
 /////////////////////////////////////////////////////////////////////
 
-bool KXMLEditorShell::openURL( const KURL & url ) const
+bool KXMLEditorShell::openURL( const KURL & url )
 {
 	if ( url.isEmpty() )
 		return false;
@@ -246,21 +192,29 @@ bool KXMLEditorShell::openURL( const KURL & url ) const
 	// says that it should open a new window if the document is _not_
 	// in its initial state.  This is what we do here.
 
+	// If there is no part yet created, create new one...
+	if (!part())
+  {
+		setPart(createXMLPart(this));
+	}
+
 	// We have to differenciate, if this shell could
 	// be used or a new one has to be created.
-	if ( m_pPart->url().isEmpty() && ! m_pPart->isModified() )
+	if ( part()->url().isEmpty() && ! part()->isModified() )
 	{
 		// this one can be used - try to open the file in it
-		if ( ! m_pPart->openURL(url) )
+		if ( ! part()->openURL(url) )
 			return false;
 	}
 	else
 	{
 		// a new one has to be created
 		KXMLEditorShell * pShell = new KXMLEditorShell;
+		// and a new part too...
+		pShell->setPart(createXMLPart(pShell));
 
 		// try to open the file in the shell
-		if ( ! pShell->m_pPart->openURL(url) )
+		if ( ! pShell->part()->openURL(url) )
 			return false;
 
 		// show the new shell
@@ -275,7 +229,7 @@ bool KXMLEditorShell::openURL( const KURL & url ) const
 
 bool KXMLEditorShell::queryClose()
 {
-	if ( ! m_pPart->closeURL() )
+	if ( part() && ! part()->closeURL() )
 		return false;
 
 	m_pActFileOpenRecent->saveEntries( instance()->config() );
@@ -285,8 +239,10 @@ bool KXMLEditorShell::queryClose()
 
 void KXMLEditorShell::saveProperties( KConfig * pConfig )
 {
-	pConfig->writeEntry( "CurrentURL", m_pPart->url().url() );
-
+	if (part())
+	{
+		pConfig->writeEntry( "CurrentURL", part()->url().url() );
+	}
 	KParts::MainWindow::saveProperties(pConfig);
 }
 
@@ -294,10 +250,10 @@ void KXMLEditorShell::readProperties( KConfig * pConfig )
 {
 	KParts::MainWindow::readProperties(pConfig);
 
-	QString strURL = pConfig->readEntry( "CurrentURL", "" );
-	if ( strURL.isEmpty())
-		return;
-	m_pPart->openURL( strURL );
+/// @todo Check if ths is a deadcode
+//	QString strURL = pConfig->readEntry( "CurrentURL", "" );
+//	if ( strURL.isEmpty())
+//	part()->openURL( strURL );
 }
 
 void KXMLEditorShell::slotEnableBrowserExtActions( const char * pszName, bool bEnabled )
@@ -307,15 +263,16 @@ void KXMLEditorShell::slotEnableBrowserExtActions( const char * pszName, bool bE
 		pAct->setEnabled( bEnabled );
 	else
 		kdDebug() << "KXMLEditor " << k_funcinfo << " unknown action (" << pszName << ")" << endl;
-
 }
 
 // Set main window caption (by using the base class functionality),
 // Enables/disables Save button (corresponding to the parts modified status)
 void KXMLEditorShell::setCaption( const QString & strCaption )
 {
-	m_pActFileSave->setEnabled( m_pPart->isModified() );
-	KParts::MainWindow::setCaption( strCaption, m_pPart->isModified() ); // base class functionality
+	if (part())
+	{
+	  KParts::MainWindow::setCaption( strCaption, part()->isModified() ); // base class functionality
+	}
 }
 
 
@@ -324,37 +281,205 @@ void KXMLEditorShell::setCaption( const QString & strCaption )
 //
 void KXMLEditorShell::slotActFileNew()
 {
+
 	// About this function, the style guide (
 	// http://developer.kde.org/documentation/standards/kde/style/basics/index.html )
 	// says that it should open a new window if the document is _not_
 	// in its initial state.  This is what we do here..
 	KXMLEditorShell * pShell;
-	if ( ! m_pPart->url().isEmpty() || m_pPart->isModified() )
+	KParts::ReadWritePart* pPart;
+
+	// 1. This shell is empty
+	if (!part())
+	{
+		pPart = createXMLPart(this);
+		pShell=this;
+	}
+	// 2. The shell contains modified part
+	else if (! part()->url().isEmpty() || part()->isModified() )
 	{
 		pShell = new KXMLEditorShell;
+		pPart = createXMLPart(pShell);
 		pShell->show();
 	}
+	// 3. The shell contains part, and it is empty and not modified
+	else
+	{
+		pPart=part();
+		pShell=this;
+	}
+
+	pShell->setPart(pPart);
 	// if there is a way to determine in KXMLEditorPart that it was created "for new file" then
 	// this part should be changed.
-	else
-		pShell = this;
-
-	((KXMLEditorPart *)pShell->m_pPart)->newFile();
+	((KXMLEditorPart *)pPart)->document()->newFile();
 }
 
 void KXMLEditorShell::mail()
 {
-	slotFileSave();							// first - our XML file have to be saved.
-	if (!m_pPart->url().isEmpty())
+	// L.V.: tried to meve save functionality to Part
+	//slotFileSave();							
+	
+	// first - our XML file have to be saved.
+	part()->save();
+	
+	if (!part()->url().isEmpty())
 	{
-		QStringList urls(m_pPart->url().url());			// we can prepare then list of attachments (1 element in the list)
-		QString theSubject(m_pPart->url().fileName(false));	// and set the mail subject
-		kapp->invokeMailer(	QString::null,			// finally we can compose e-mail (addressed to no-one) out of that
+		QStringList urls(part()->url().url());			// we can prepare then list of attachments (1 element in the list)
+		QString theSubject(part()->url().fileName(false));	// and set the mail subject
+		kapp->invokeMailer(QString::null,				// finally we can compose e-mail (addressed to no-one) out of that
 						QString::null,
 						QString::null,
 						theSubject,				// subject
-						QString::null,			// body
+						QString::null,				// body
 						QString::null,
-						urls);					// attachments
+						urls);						// attachments
 	}
 }
+
+void KXMLEditorShell::setPart(KParts::ReadWritePart* pPart)
+{
+	if (pPart==m_pPart)
+		return;	// nothing to do here...
+	else
+	{
+		if(pPart)
+		{
+			connect(static_cast <KXMLEditorPart *> (pPart), SIGNAL(sigAddRecentURL(const KURL &)), 
+							this, SLOT(slotAddRecentURL(const KURL &)) );
+		}
+	}
+
+	if (m_pPart)
+	{
+		factory()->removeClient(m_pPart);
+		delete m_pPart;
+	}
+	m_pPart = pPart;
+	if (pPart)
+	{
+		// get the parts browser extension and connect additional actions to it
+		KParts::BrowserExtension * pBrowserExt = 0;
+		if ( (pBrowserExt = KParts::BrowserExtension::childObject(pPart)) )  // if there is one
+		{
+			connect( pBrowserExt, SIGNAL(enableAction( const char *, bool )), this, SLOT(slotEnableBrowserExtActions( const char *, bool )) );
+
+			// L.V. moved to part. connect(pActPrint,SIGNAL(activated()),pBrowserExt, SLOT(print()));
+			//pActPrint->setEnabled(pBrowserExt->isActionEnabled("print"));
+		}
+
+		createGUI(pPart);
+		setCentralWidget(pPart->widget() );
+		pPart->widget()->show();
+		stateChanged("part");
+	}
+	else
+	{
+		createGUI(0L);
+		setCentralWidget(0L);
+		stateChanged("empty");
+	}
+
+	// keep pointer to window menu
+	m_windowMenu = static_cast<KPopupMenu*>(factory()->container("window", this));
+	if(!m_windowMenu)
+		kdDebug() << "KXMLEditorShell::setPart - m_windowMenu is NULL" << endl;
+	else
+		connect(m_windowMenu,SIGNAL(aboutToShow()),this,SLOT(slotBeforeWindowMenuShown()));
+}
+
+KParts::ReadWritePart* KXMLEditorShell::createXMLPart(QObject *parent)
+{
+	return  createPart(parent,"libkxmleditorpart", "KParts::ReadWritePart");
+}
+
+KParts::ReadWritePart* KXMLEditorShell::createPart(QObject *parent, const QString& libName, const QString& className)
+{
+	KParts::ReadWritePart* newPart = 0L;
+	KLibFactory * pFactory = KLibLoader::self()->factory(libName);
+	if (pFactory)
+		newPart = static_cast <KParts::ReadWritePart*> ( pFactory->create( parent, "KXMLEditor view", className ) );
+	else
+		kdFatal() << "KXMLEditorShell::KXMLEditorShell no " << libName << " found" << endl;
+	return newPart;
+}
+
+void KXMLEditorShell::slotActClose()
+{
+	if (part() && part()->queryClose())
+		setPart(0L);
+}
+
+void KXMLEditorShell::close()
+{
+	if (queryClose())
+	{
+		if (KMainWindow::memberList->count()==1)
+			kapp->deref();
+		delete this;
+	}
+}
+
+void KXMLEditorShell::slotActWindowCloseAllOthers()
+{
+	QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
+	for (it.toFirst();it.current();++it)
+	{
+		if (it.current()!=this)
+		it.current()->close();
+	}
+}
+
+    void KXMLEditorShell::slotActWindowClose()
+{
+	this->close();
+}
+
+void KXMLEditorShell::slotActWindowCloseAll()
+{
+	slotActWindowCloseAllOthers();
+	slotActWindowClose();
+}
+
+KXEShellManager* KXMLEditorShell::manager()
+{
+	if (!s_manager)
+	{
+		s_manager = new KXEShellManager();
+	}
+	return s_manager;
+}
+
+void KXMLEditorShell::slotBeforeWindowMenuShown()
+{
+	// first we have to clean all shell items...
+	m_windowMenu->clear();
+
+	// then we plug-in neccessary actions...
+	pActCloseW->plug(m_windowMenu);
+	pActCloseAllW->plug(m_windowMenu);
+	pActCloseAllO->plug(m_windowMenu);
+	m_windowMenu->insertSeparator();
+
+	// and then we can add entries for all shells
+	// that are registered in Shell Manager...
+	QPtrListIterator<KXMLEditorShell> it (*(manager()->shells()));
+	for (it.toFirst(); it.current();++it)
+	{
+		int id = m_windowMenu->insertItem(it.current()->caption(),it.current(),SLOT(slotActivate()));
+		m_windowMenu->setItemChecked(id,it.current()==this);
+	}
+}
+
+void KXMLEditorShell::slotActivate()
+{
+	/// @todo windows have to be activated over EWMH
+	showNormal();			// show it if it's minimized
+	raise();				// bring that to front
+}
+
+// Add URL to recent file list
+void KXMLEditorShell::slotAddRecentURL(const KURL &url)
+{
+	m_pActFileOpenRecent->addURL(url);
+}
