@@ -33,7 +33,9 @@ KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pP
 	: QListViewItem(pParent),
 	  m_xmlNode(xmlNode),
 	  m_pPrevSibling(0),
-	  m_bBookmarked(false)
+	  m_bBookmarked(false),
+	  m_bChildsCreated(false),
+	  m_bGrandChildsCreated(false)
 {
 	init();
 	initChilds();
@@ -42,8 +44,10 @@ KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pP
 KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, KListView * pParent,  QListViewItem* pAfter)
 	: QListViewItem(pParent,pAfter),
 	  m_xmlNode(xmlNode),
-    m_pPrevSibling(0),
-	  m_bBookmarked(false)
+	  m_pPrevSibling(0),
+	  m_bBookmarked(false),
+	  m_bChildsCreated(false),
+	  m_bGrandChildsCreated(false)
 {
 	init();
 	initChilds();
@@ -52,8 +56,10 @@ KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, KListView * pParen
 KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pParent, QListViewItem * pAfter )
 	: QListViewItem( pParent, pAfter ),
 	  m_xmlNode(xmlNode),
-    m_pPrevSibling(0),
-	  m_bBookmarked(false)
+	  m_pPrevSibling(0),
+	  m_bBookmarked(false),
+	  m_bChildsCreated(false),
+	  m_bGrandChildsCreated(false)
 {
 	init();
 	initChilds();
@@ -138,13 +144,12 @@ void KXE_TreeViewItem::setTexts()
 
 void KXE_TreeViewItem::initChilds()
 {
-	QDomNode tmpNode = m_xmlNode.lastChild();
-
-	while ( ! tmpNode.isNull() )
-	{
-		new KXE_TreeViewItem( tmpNode, this );
-		tmpNode = tmpNode.previousSibling();		
-	}
+	if ( ! KXMLEditorFactory::configuration()->treeview()->createItemsOnDemand() ||
+	     ( ! m_bChildsCreated && 
+	       ( ! parent() || ( parent() && parent()->isOpen() ) ) 
+	     ) 
+	   )
+		ensureChildItemsCreated();
 }
 
 bool KXE_TreeViewItem::toggleBookmark()
@@ -270,3 +275,43 @@ bool KXE_TreeViewItem::isMyChildren(const KXE_TreeViewItem *pTestItem)
     }
 	return false;
 }
+
+
+void KXE_TreeViewItem::ensureChildItemsCreated()
+{
+	if ( ! m_bChildsCreated )
+	{
+		QDomNode tmpNode = m_xmlNode.lastChild();
+
+		while ( ! tmpNode.isNull() )
+		{
+			new KXE_TreeViewItem( tmpNode, this );
+			tmpNode = tmpNode.previousSibling();		
+		}
+
+		m_bChildsCreated = true;
+	}
+}
+
+
+void KXE_TreeViewItem::ensureGrandChildItemsCreated()
+{
+	if ( ! m_bGrandChildsCreated )
+	{
+		// ensure, that all child items of this item are created
+		if ( ! m_bChildsCreated )
+			ensureChildItemsCreated();
+
+		// Iterate over all children now and ensure their child items 
+		// (this' grandchildrens) are created.
+		KXE_TreeViewItem * pChild = static_cast <KXE_TreeViewItem*> ( firstChild() );
+		while ( pChild )
+		{
+			pChild->ensureChildItemsCreated();
+			pChild = static_cast <KXE_TreeViewItem*> ( pChild->nextSibling() );
+		}
+
+		m_bGrandChildsCreated = true;
+	}
+}
+
