@@ -91,11 +91,24 @@ class KXE_TreeViewItem : public QListViewItem
 		/** Test, if item in parameter is my direct or indirect child item */
   bool isMyChildren(const KXE_TreeViewItem *);
 
+		/** 
+		 * If the child items aren't created (initialized) yet
+		 * (i.e. if m_bChildsCreated is false), it is done now.
+		 */
+		void ensureChildItemsCreated();
+		/** 
+		 * If the grandchilds (child items of this item's childs) aren't created (initialized)
+		 * yet (i.e. if m_bGrandChildsCreated is false), it is done now.
+		 */
+		void ensureGrandChildItemsCreated();
+  
 protected:
 
 		QDomNode m_xmlNode;
 		KXE_TreeViewItem * m_pPrevSibling;
 		bool m_bBookmarked;
+		bool m_bChildsCreated;
+		bool m_bGrandChildsCreated;
 
 private:
 
