@@ -22,9 +22,12 @@
 #include <kparts/browserextension.h>
 
 #include <qdom.h>
+#include <qptrlist.h>
 
 #include "kxe_treeview.h"
 
+#define FILE_DIALOG_FILTER "*.xml|XML files\n*.xsl|XSL files\n*.xslt|XSLT files\n*.svg|SVG files\n*.xul|XUL files\n*.rc|Resource UI files\n*.ui|User Interface UI files\n*|All files"
+
 class QTabWidget;
 class KXE_ViewElement;
 class KXESearchDialog;
@@ -38,23 +41,50 @@ class KXMLEditorPartIfaceReadOnly; // DCOP interface
 class KCommandHistory;
 class KPrinter;
 class QSplitter;
+class KXEDocument;
 
-/** This is the KPart of KXMLEditor. */
+
+/** @short This is the KPart of KXMLEditor. */
 class KXMLEditorPart : public KParts::ReadWritePart
 {
 		Q_OBJECT
 
 public:
-		/** Creates the actions, the view (widgets) and changes to "begin state". */
-		KXMLEditorPart( bool fReadWrite, QWidget * pParent, const char * pszName );
+		/** @short Constructor
+
+		Creates the actions, the view (widgets) and changes to "begin state".
+		*/
+		KXMLEditorPart( bool fReadWrite, KXEDocument* pDocument, QWidget * pParent, const char * pszName );
+
+		/** @short Destructor */
 		virtual ~KXMLEditorPart();
 
-		/** Changes the behaviour of this part to readonly or readwrite. */
+		/** @short Changes the behaviour of this part to readonly or readwrite. */
 		virtual void setReadWrite( bool fReadWrite = true );
 
-		/** Returns the selected XML object's path or an empty string, if no object is selected. */
+		/** @short Returns the selected XML object's path or an empty string, if no object is selected. */
 		QString getSelectedPath() const { return m_pViewTree->getSelectedPath(); }
 
+		/** @short Copy XML node into clipboard */
+		QTextDrag * copyNode(QDomNode *);
+
+		/** @short Paste XML node from clipboard into document */
+		bool pasteNode(QDomNode *, QMimeSource *);
+
+		/** @short Drag&Drop move */
+		bool dropMoveNode(QDomElement &, QDomNode &);
+
+		/** @short Prints the document on specified printer.
+			@param pPrinter printer which will be used for printing. */
+		virtual void print(KPrinter* pPrinter);
+
+		/** @short Returns underlying document. */
+		KXEDocument* document()  	{ return m_pDocument; }
+
+		/** @short Changes underlying document object */
+		void setDocument(KXEDocument *pDocument);
+
+public slots:
 		/**
 		 * Reimplemented from @ref KParts::KReadWritePart::setModified.
 		 *
@@ -63,222 +93,178 @@ public:
 		 */
 		virtual void setModified( bool fModified );
 
-		/** Copy XML node into clipboard */
-		QTextDrag * copyNode(QDomNode *);
-
-		/** Paste XML node from clipboard into document */
-		bool pasteNode(QDomNode *, QMimeSource *);
-
-    /** Drag&Drop move */
-    bool dropMoveNode(QDomElement &, QDomNode &);
-
-		/**    Prepares content of a newly created XML file. */
-		virtual void newFile();
-
-		/** Updates tree view after creating new element */
+		/** @short Updates tree view after creating new element */
 		void updateNodeCreated(const QDomNode & node);
 
-		/** Updates tree view after deleting new element */
+		/** @short Updates tree view after deleting new element */
 		void updateNodeDeleted(const QDomNode & node);
 
-		/** Updates tree view after change element properties */
+		/** @short Updates tree view after change element properties */
 		void updateNodeChanged( const QDomElement & domElement );
 
-    /** Updates tree view after change char. data properties */
-    void updateNodeChanged( const QDomCharacterData & node ) ;
+		/** @short Updates tree view after change char. data properties */
+		void updateNodeChanged( const QDomCharacterData & node ) ;
 
-    /** Updates tree view after change proc. instr. properties */
-    void updateNodeChanged( const QDomProcessingInstruction &domProcInstr );
+		/** @short Updates tree view after change proc. instr. properties */
+		void updateNodeChanged( const QDomProcessingInstruction &domProcInstr );
 
-		/** Updates tree view after move node */
+		/** @short Updates tree view after move node */
 		void updateNodeMoved( const QDomNode & node );
 
-		/** Attaches stylesheet to current XML file. If file is already attached to stylesheet,
-		this attachment will be overriten.
-		@param stylesheet url to stylesheet location
-		@sa detachStylesheet() */
-		virtual void attachStylesheet(const KURL& stylesheet);
-
-    /** Detaches stylesheet from current XML file.
-		@sa attachStylesheet()*/
-		virtual void detachStylesheet();
-
-    /** Attaches schema to current XML file. If file is already attached to schema,
-		this attachment will be overriten.
-		@param schema url to schema location
-		@sa detachSchema()*/
-		virtual void attachSchema(const KURL& schema);
-
-    /** Detaches schema from current XML file.
-		@sa attachSchema()*/
-		virtual void detachSchema();
-		/** Prints the document on specified printer.
-			@param pPrinter printer which will be used for printing. */
-		virtual void print(KPrinter* pPrinter);
-		/** Adds or changes (if already exists) processing instruction. Processing
-		instruction will be added as a child in QDomDocument object.
-			@param target processing intruction target
-			@param data processing intruction data */
-		virtual void setSpecProcInstr(const QString& target, const QString& data);
-
-public slots:
-
 		/**
-		 * Reimplemented from @ref KParts::KReadWritePart::setModified.
+		 * @short Reimplemented from @ref KParts::KReadWritePart::setModified.
 		 */
-		virtual void setModified() { setModified(true); updateActions(); }
+		virtual void setModified() { setModified(true);  }
+		
+		/** @short Slot connected to File->Save As action. */
+		bool slotFileSaveAs();
 
-		/** Cut the marked text/object into the clipboard */
-		void slotEditCut();
-
-    /** Copy the marked text/object into the clipboard */
-		void slotEditCopy();
-
-    /** Paste the clipboard into the document */
-		void slotEditPaste();
-
-    /** Displays search dialog and finds string in this tree view */
-		void slotEditFind();
-
-    /** Finds next occurence of string in this tree view */
-		void slotEditFindNext();
-
-    /** Deselects currently selected item */
-		void slotEditDeselect() { m_pViewTree->editDeselect(); }
-
-    /** Selects the selected item's parent item. */
-		void slotViewNodeUp() { m_pViewTree->viewNodeUp(); }
-
-    /** Expands the selected tree item's subtree. */
-		void slotViewExpNode() { m_pViewTree->viewExpNode(-1); }
-
-    /** Expands the selected tree item's subtree to the given level. */
-		void slotViewExpNode( int nLevel ) { m_pViewTree->viewExpNode(nLevel); }
-
-    /** Collapses the selected tree item's subtree. */
-		void slotViewColNode() { m_pViewTree->viewColNode(0); }
-
-    /** Collapses the selected tree item's subtree to the given level. */
-		void slotViewColNode( int nLevel ) { m_pViewTree->viewColNode(nLevel); }
-
-    /** Insert XML element into document */
-		void slotXmlElementInsert();
-
-    /** Edit XML element */
-		void slotXmlElementEdit();
-
-    /** Add attribute to an XML element. */
-		void slotXmlAttributesAdd();
-
-    /** Delete all attributes of an XML element. */
-		void slotXmlAttributesDel();
-
-    /** Delete an attribute of an XML element. */
-		void slotXmlAttributeDel();
-
-    /** Insert proc.instruction into document */
-		void slotXmlProcInstrInsert();
-
-    /** Edit proc.instruction */
-		void slotXmlProcInstrEdit();
-
-    /** Insert special proc.instruction into document */
+		/** @short Inserts special proc.instruction into document. */
 		void slotActVersionEncoding();
 
-    /* Insert text into element. */
+		/** @short Called on user's action response.
+		Creates <?xml-stylesheet ...?> processing instruction
+		with reference to specified stylesheet URI.
+		*/
+		void slotActAttachStylesheet();
+	
+		/** @short Called on user's action response. */
+		void slotActDetachStylesheet();
+	
+		/** @short Called on user's action response */
+		void slotActDetachSchema();
+	
+		/** @short Called on user's action response */
+		void slotActAttachSchema();
+
+		/** @short Cut the marked text/object into the clipboard */
+		void slotEditCut();
+
+		/** @short Copy the marked text/object into the clipboard */
+		void slotEditCopy();
+
+		/** @short Paste the clipboard into the document */
+		void slotEditPaste();
+
+		/** @short Displays search dialog and finds string in this tree view */
+		void slotEditFind();
+
+		/** @short Finds next occurence of string in this tree view */
+		void slotEditFindNext();
+
+		/** @short Deselects currently selected item */
+		void slotEditDeselect() { m_pViewTree->editDeselect(); }
+
+		/** @short Selects the selected item's parent item. */
+		void slotViewNodeUp() { m_pViewTree->viewNodeUp(); }
+
+		/** @short Expands the selected tree item's subtree. */
+		void slotViewExpNode() { m_pViewTree->viewExpNode(-1); }
+
+		/** @short Expands the selected tree item's subtree to the given level. */
+		void slotViewExpNode( int nLevel ) { m_pViewTree->viewExpNode(nLevel); }
+
+		/** @short Collapses the selected tree item's subtree. */
+		void slotViewColNode() { m_pViewTree->viewColNode(0); }
+
+		/** @short Collapses the selected tree item's subtree to the given level. */
+		void slotViewColNode( int nLevel ) { m_pViewTree->viewColNode(nLevel); }
+
+		/** @short Insert XML element into document */
+		void slotXmlElementInsert();
+
+		/** @short Edit XML element */
+		void slotXmlElementEdit();
+
+		/** @short Add attribute to an XML element. */
+		void slotXmlAttributesAdd();
+
+		/** @short Delete all attributes of an XML element. */
+		void slotXmlAttributesDel();
+
+		/** @short Delete an attribute of an XML element. */
+		void slotXmlAttributeDel();
+
+		/** @short Insert proc.instruction into document */
+		void slotXmlProcInstrInsert();
+
+		/** @short Edit proc.instruction */
+		void slotXmlProcInstrEdit();
+
+		/* @short Insert text into element. */
 		void slotActInsertText();
-    /* Insert CDATA section. */
+
+		/* @short Insert CDATA section. */
 		void slotActInsertCDATA();
-    /* Insert comment to element. */
+
+		/* @short Insert comment to element. */
 		void slotActInsertComment();
 
-    /* Edit character data */
+		/* @short Edit character data */
 		void slotXmlCharDataEdit();
 
-    /* Moves a node up (change its position with its prev. siblings position) */
+		/* @short Moves a node up (change its position with its prev. siblings position) */
 		void slotXmlMoveNodeUp();
 
-    /* Moves a node down (change its position with its next siblings position) */
+		/* @short Moves a node down (change its position with its next siblings position) */
 		void slotXmlMoveNodeDown();
 
-    /** toggles bookmark on the selected item */
+		/** @short toggles bookmark on the selected item */
 		void slotBookmarksToggle();
 
-    /** searches for the previous bookmarked item */
+		/** @short Searches for the previous bookmarked item */
 		void slotBookmarksPrev() { m_pViewTree->bookmarksPrev(); }
 
-    /** searches for the next bookmarked item */
+		/** @short Searches for the next bookmarked item */
 		void slotBookmarksNext() { m_pViewTree->bookmarksNext(); }
 
-    /** shows configuration dialog (@ref OptionsDialog) */
+		/** @short Shows configuration dialog (@ref OptionsDialog) */
 		void slotConfigure();
 
 		void slotPathSelected( const QString & szPath );
 		void slotPathSelected();
 		void slotPathClear();
-		/**
-		 * Called when a tree view items text was changed via inplace-renaming.
-		 */
+
+		/** @short Called when a tree view items text was changed via inplace-renaming. */
 		void slotItemRenamedInplace( QListViewItem * pItem );
 
-    /** Called whan an attributes name was changed via inplace editing. */
+		/** @short Called whan an attributes name was changed via inplace editing. */
 		void slotAttributeNameChangedInplace( const QDomAttr &, const QString );
 
-    /** Called whan an attributes value was changed via inplace editing. */
+		/** @short Called whan an attributes value was changed via inplace editing. */
 		void slotAttributeValueChangedInplace( const QDomAttr &, const QString );
 
-    /** Called on user's action response. Creates <?xml-stylesheet ...?> processing instruction
-		with reference to specified stylesheet URI.
-		*/
-		void slotActAttachStylesheet();
-
-    /** Called on user's action response. */
-		void slotActDetachStylesheet();
-
-    /** Called on user's action response */
-		/** Called when File->Print or icon from the toolbar is selected. */
+		/** @short Called when File->Print or icon from the toolbar is selected. */
 		void slotActPrint();
 
-		void slotActDetachSchema();
-
-		/** Called on user's action response */
-
-		void slotActAttachSchema();
-
+		/** @short Slot for editing node properties. */
 		void slotActProperties();
 
-    void slotActEditRawXml();
+		/** @short Slot for editing xml as a text in separate dialog window. */
+		void slotActEditRawXml();
 
+		/** @short Slot for removing XML nodes. */
 		void slotActDelete();
 
 		void slotTreeViewKeyPressed(QKeyEvent *e);
 
+		/** @short Slot that performs redo function. */
+		void slotActRedo();
+
+		/** @short Slot that performs undo function. */
+		void slotActUndo();
+
 protected:
-		/** Opens the file @ref KParts::ReadOnlyPart::m_file. */
+
+		void updateActions();
+		
+		/** @short Opens the file @ref KParts::ReadOnlyPart::m_file. */
 		virtual bool openFile();
 
-    /** Saves the file @ref KParts::ReadOnlyPart::m_file. */
+		/** @short Saves the file @ref KParts::ReadOnlyPart::m_file. */
 		virtual bool saveFile();
 
-    /** Removes spec. processing instruction from XML file.
-			@param target name of the proc. instruction target
-			@warning Removes processing instruction from DomDocument children node list only (no subnodes are modiified).
-		*/
-		void removeSpecProcInstr(const QString &target);
-
-    /** Returns processing instruction located in QDomDocument. If specified
-		processing intruction is not found, null node is returned.
-			@param target name of the processing instruction target. */
-		QDomNode getSpecProcInstr(const QString& target);
-
-    /** Updates status of actions (enabled/disabled). Is it automatically invoked everytime content is modified.
-		Good place for updating actions which do not depend on current selection, but are file content dependent,
-		like #m_pActXmlAttachStylesheet #m_pActXmlAttachSchema.
-			@sa setModified()
-		*/
-		void updateActions();
-
-    /** Draws header for printouts (made of file name).
+		/** @short Draws header for printouts (made of file name).
 			@param painter a painter to use for drawing
 			@param pageNumber number of currently drawn page counted from 0
 			@param ypos y coordinate of upper border of the header
@@ -286,7 +272,7 @@ protected:
 		*/
 		void printHeader(QPainter* painter, int pageNumber, int ypos, int width);
 
-    /** Draws footer for printouts (page number).
+		/** @short Draws footer for printouts (page number).
 			@param painter a painter to use for drawing
 			@param pageNumber number of currently drawn page counted from 0
 			@param ypos y coordinate of upper border of the footer
@@ -294,7 +280,9 @@ protected:
 		*/
 		void printFooter(QPainter* painter,int pageNumber, int ypos, int width);
 
-    /** Draws page for printing purposes. If there are more pages to print, method
+		/** @short Draws page for printing purposes.
+
+		If there are more pages to print, method
 		returns true. Otherwise returns false.
 			@param painter a painter to use for srawing
 			@param pageNumber number of currently drawn page counted from 0
@@ -304,9 +292,10 @@ protected:
 		*/
 		bool printPage(QPainter* painter,int pageNumber, int top, int width, int height);
 
-		/** Used in printing function printPage(). */
+		/** @short Used in printing function printPage(). */
 		unsigned int m_printLineNumber;
-		/** Used in printing function printPage(). */
+
+		/** @short Used in printing function printPage(). */
 		QStringList m_printLines;
 
 		// the actions
@@ -331,101 +320,107 @@ protected:
 		KAction * m_pActBookmarksNext;
 		KXmlEditorComboAction * m_pActPathCombo;
 
-		/** the document */
-		QDomDocument * m_pDoc;
-
 	protected slots:
 		void started();
 		void completed();
 		void canceled();
 
-    /** enables/disables actions and changes views  */
+		/** @short Enables/disables actions and changes views  */
 		void slotSelectionCleared(bool);
 
-    /** enables/disables actions depending on the given XML element and changes views  */
+		/** @short Enables/disables actions depending on the given XML element and changes views  */
 		void slotSelectionChanged( const QDomElement & );
 
-    /** enables/disables actions and changes views */
+		/** @short Enables/disables actions and changes views */
 		void slotSelectionChanged( const QDomCharacterData & );
 
-    /** enables/disables actions and changes views */
+		/** @short Enables/disables actions and changes views */
 		void slotSelectionChanged( const QDomProcessingInstruction & );
 
-		/** Shows the requested context menu at the given position. */
+		/** @short Shows the requested context menu at the given position. */
 		void slotContextMenuRequested( const QString & szMenuName, const QPoint & pos );
 
+		/** @short Invoked everytime document has new content */
+		void slotDocOpened();
+
 	private:
 		/** this view displays the XML-document's tree structure */
 		KXE_TreeView * m_pViewTree;
 
-    /** the tab widget, contains: m_pViewElement, m_pViewContents */
+		/** the tab widget, contains: m_pViewElement, m_pViewContents */
 		QTabWidget * m_pTabWidget;
 
-    /** widget to display XML element */
+		/** widget to display XML element */
 		KXE_ViewElement * m_pViewElement;
 
-    /** edit widget for contents */
+		/** edit widget for contents */
 		QTextEdit * m_pViewContents;
 
-    /** widget to display a proc.instr. */
+		/** widget to display a proc.instr. */
 		QTextEdit * m_pViewProcInstr;
 
-    /** the search dialog, used for find action */
+		/** the search dialog, used for find action */
 		KXESearchDialog * m_pDlgSearch;
 
-	protected:
 		/** DCOP object having access to XML document. It may by NULL if no DCOP inteface is provided
-		* or read only privided by @ref KXMLEditorPartReadOnlyIface or full read-write @ref KXMLEditorPartIface .
-		*/
+		or read only privided by @ref KXMLEditorPartReadOnlyIface or full read-write @ref KXMLEditorPartIface .*/
 		KXMLEditorPartIfaceReadOnly * m_pDCOPIface;
 
 		bool m_bAlreadyModified;
 
 		/** our browser extension */
 		KParts::BrowserExtension * m_pBrowserExt;
+		
+		/** @short Stores pointer to command history list, which provide undo/redo functionality */
+		KCommandHistory* m_pCmdHistory;
+		
+		/** @short Action for cutting nodes to clipboard.*/
+		KAction* m_pActEditCut;
 
-		// Indicates, if document is opened from tar.gz
-		bool m_bDocIsCompressed;
+		/** @short Action for copying nodes to clipboard.*/
+		KAction* m_pActEditCopy;
 
-		/** Name of entry in Tar archive */
-		QString m_strCompressedTarEntryName;
+		/** @short Action for pasting nodes from clipboard.*/
+		KAction* m_pActEditPaste;
 
-    /** Action for changing version and encoding of XM file. */
-		KAction* m_pActVersionEncoding;
-
-    /** Action for attaching stylehseets to xml files. */
-		KAction* m_pActAttachStylesheet;
-
-    /** Action for detaching stylehseets from xml files. */
-		KAction* m_pActDetachStylesheet;
-
-    /** Action for attaching schemas to xml files. */
-		KAction* m_pActAttachSchema;
-
-    /** Action for detaching schemas from xml files. */
-		KAction* m_pActDetachSchema;
-
-    /** Action for editing properties of currently selected node. */
+		/** Action for editing properties of currently selected node. */
 		KAction* m_pActProperties;
 
-    /** Action for editing raw XML of currently selected node. */
+		/** Action for editing raw XML of currently selected node. */
 		KAction* m_pActEditRawXml;
 
-    /** Action for deleting currently selected node. */
+		/** Action for deleting currently selected node. */
 		KAction* m_pActDelete;
 
-    /** Printer object. */
+		/** @short Action for changing version and encoding of XM file. */
+		KAction* m_pActVersionEncoding;
+
+		/** @short Action for attaching stylehseets to xml files. */
+		KAction* m_pActAttachStylesheet;
+
+		/** @short Action for detaching stylehseets from xml files. */
+		KAction* m_pActDetachStylesheet;
+
+		/** @short Action for attaching schemas to xml files. */
+		KAction* m_pActAttachSchema;
+
+		/** @short Action for detaching schemas from xml files. */
+		KAction* m_pActDetachSchema;
+		
+		/** Printer object. */
 		KPrinter *m_pPrinter;
 
-    /** Command history used to implement Undo/Redo functionality. */
-		KCommandHistory* m_pCmdHistory;
+		/** spitter - main widget in the part */
+		QSplitter *pSplitter;
 
-	/* spitter - main widget in the part */
-	QSplitter *pSplitter;
+		/** Pointer to underlying KXEDocument object */
+		KXEDocument* m_pDocument;
+		
+signals:
+		// Add URL to recent file list
+		void sigAddRecentURL(const KURL &);
 };
 
-
-
 /**
  * The browser extension for our part (@ref KXMLEditorPart).
  */
@@ -434,16 +429,20 @@ class KXMLEditorBrowserExtension : public KParts::BrowserExtension
 		Q_OBJECT
 
 	public:
-		KXMLEditorBrowserExtension( KXMLEditorPart * pParent, const char * pszName = "KXMLEditorBrowserExtension" ) : KParts::BrowserExtension( pParent, pszName ), m_pPart(pParent)
-		{ emit enableAction("print",true); }
+		/** @short Constructor */
+		KXMLEditorBrowserExtension( KXMLEditorPart * pParent, const char * pszName = "KXMLEditorBrowserExtension" )
+			: KParts::BrowserExtension( pParent, pszName ),
+			m_pPart(pParent)
+			{ emit enableAction("print", true); }
 
 	public slots:
-		void cut() { m_pPart->slotEditCut(); }
-		void copy() { m_pPart->slotEditCopy(); }
-		void paste() { m_pPart->slotEditPaste(); }
+		void slotEditCut() { m_pPart->slotEditCut(); }
+		void slotEditCopy() { m_pPart->slotEditCopy(); }
+		void slotEditPaste() { m_pPart->slotEditPaste(); }
 		void print() { m_pPart->slotActPrint(); }
 
 	protected:
+		/** @short Stores pointer to the part. */
 		KXMLEditorPart * m_pPart;
 };
 
